Package: sobol
Type: Package
Title: Quasi-Monte Carlo Sobol Sequence Generator
Version: 1.0.0
Authors@R: c(
    person("Angel", "Robles", email = "a.l.robles.fernandez@gmail.com", role = c("aut", "cre")),
    person("Ilya M.", "Sobol", role = "ctb", comment = "Original Sobol sequence algorithm"),
    person("Paul", "Bratley", role = "ctb", comment = "Algorithm implementation reference"),
    person("Bennett L.", "Fox", role = "ctb", comment = "Algorithm implementation reference"),
    person("Stephen", "Joe", role = "ctb", comment = "Direction numbers and primitive polynomials"),
    person("Frances Y.", "Kuo", role = "ctb", comment = "Direction numbers and primitive polynomials")
    )
Description: Provides a fast and efficient implementation of Sobol sequences
    for quasi-Monte Carlo methods. The Sobol sequence is a low-discrepancy
    sequence with the property that for all values of N, its subsequence
    x1, ..., xN has a low discrepancy. It can be used to generate
    quasi-random numbers for use in Monte Carlo integration and other
    simulation methods. This implementation is based on the algorithms
    described by Bratley and Fox (1988) <doi:10.1145/42288.214372> and
    uses direction numbers from Joe and Kuo (2008) <doi:10.1145/1358628.1358630>.
    The package includes both batch and incremental interfaces with support
    for arbitrary starting indices and reproducible sequences. It uses 'Rcpp'
    for efficient 'C++' integration.
URL: https://alrobles.github.io/sobol/
BugReports: https://github.com/alrobles/sobol/issues
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.1.0), methods, checkmate
LinkingTo: Rcpp
Suggests: testthat (>= 3.0.0), ggplot2, microbenchmark
RoxygenNote: 7.3.3
License: GPL (>= 3)
NeedsCompilation: yes
Packaged: 2026-05-13 04:35:08 UTC; alrob
Author: Angel Robles [aut, cre],
  Ilya M. Sobol [ctb] (Original Sobol sequence algorithm),
  Paul Bratley [ctb] (Algorithm implementation reference),
  Bennett L. Fox [ctb] (Algorithm implementation reference),
  Stephen Joe [ctb] (Direction numbers and primitive polynomials),
  Frances Y. Kuo [ctb] (Direction numbers and primitive polynomials)
Maintainer: Angel Robles <a.l.robles.fernandez@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-18 18:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-18 19:11:10 UTC; unix
Archs: sobol.so.dSYM

Package: sobol
Type: Package
Title: Quasi-Monte Carlo Sobol Sequence Generator
Version: 1.0.0
Authors@R: c(
    person("Angel", "Robles", email = "a.l.robles.fernandez@gmail.com", role = c("aut", "cre")),
    person("Ilya M.", "Sobol", role = "ctb", comment = "Original Sobol sequence algorithm"),
    person("Paul", "Bratley", role = "ctb", comment = "Algorithm implementation reference"),
    person("Bennett L.", "Fox", role = "ctb", comment = "Algorithm implementation reference"),
    person("Stephen", "Joe", role = "ctb", comment = "Direction numbers and primitive polynomials"),
    person("Frances Y.", "Kuo", role = "ctb", comment = "Direction numbers and primitive polynomials")
    )
Description: Provides a fast and efficient implementation of Sobol sequences
    for quasi-Monte Carlo methods. The Sobol sequence is a low-discrepancy
    sequence with the property that for all values of N, its subsequence
    x1, ..., xN has a low discrepancy. It can be used to generate
    quasi-random numbers for use in Monte Carlo integration and other
    simulation methods. This implementation is based on the algorithms
    described by Bratley and Fox (1988) <doi:10.1145/42288.214372> and
    uses direction numbers from Joe and Kuo (2008) <doi:10.1145/1358628.1358630>.
    The package includes both batch and incremental interfaces with support
    for arbitrary starting indices and reproducible sequences. It uses 'Rcpp'
    for efficient 'C++' integration.
URL: https://alrobles.github.io/sobol/
BugReports: https://github.com/alrobles/sobol/issues
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.1.0), methods, checkmate
LinkingTo: Rcpp
Suggests: testthat (>= 3.0.0), ggplot2, microbenchmark
RoxygenNote: 7.3.3
License: GPL (>= 3)
NeedsCompilation: yes
Packaged: 2026-05-13 04:35:08 UTC; alrob
Author: Angel Robles [aut, cre],
  Ilya M. Sobol [ctb] (Original Sobol sequence algorithm),
  Paul Bratley [ctb] (Algorithm implementation reference),
  Bennett L. Fox [ctb] (Algorithm implementation reference),
  Stephen Joe [ctb] (Direction numbers and primitive polynomials),
  Frances Y. Kuo [ctb] (Direction numbers and primitive polynomials)
Maintainer: Angel Robles <a.l.robles.fernandez@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-18 18:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-18 18:54:03 UTC; unix
Archs: sobol.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,sobol_generator)
S3method(print,summary.sobol_generator)
S3method(summary,sobol_generator)
export(sobol_design)
export(sobol_dimensions)
export(sobol_generator)
export(sobol_index)
export(sobol_next)
export(sobol_next_n)
export(sobol_points)
export(sobol_skip_to)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
useDynLib(sobol, .registration = TRUE)

# Performance Profiling Script for Sobol Library
# This script profiles specific operations to identify bottlenecks

library(sobol)

# Create a profiling function
profile_operation <- function(name, func, times = 100) {
  cat("\nProfiling:", name, "\n")

  # Warmup
  invisible(func())

  # Time execution
  start_time <- Sys.time()
  for (i in seq_len(times)) {
    invisible(func())
  }
  end_time <- Sys.time()

  elapsed <- as.numeric(end_time - start_time, units = "secs")
  avg_time <- elapsed / times

  cat(sprintf("  Total time: %.4f seconds\n", elapsed))
  cat(sprintf("  Average time per call: %.4f seconds\n", avg_time))
  cat(sprintf("  Operations per second: %.2f\n", times / elapsed))

  return(list(
    name = name,
    total_time = elapsed,
    avg_time = avg_time,
    ops_per_sec = times / elapsed
  ))
}

# Profile different operations
results <- list()

# 1. Small dimension, small batch
results$small_small <- profile_operation(
  "Small dim (2), small batch (100)",
  function() sobol_points(n = 100, dim = 2)
)

# 2. Small dimension, large batch
results$small_large <- profile_operation(
  "Small dim (2), large batch (100000)",
  function() sobol_points(n = 100000, dim = 2),
  times = 10
)

# 3. Medium dimension, medium batch
results$medium_medium <- profile_operation(
  "Medium dim (10), medium batch (10000)",
  function() sobol_points(n = 10000, dim = 10),
  times = 20
)

# 4. High dimension, small batch
results$high_small <- profile_operation(
  "High dim (100), small batch (1000)",
  function() sobol_points(n = 1000, dim = 100),
  times = 20
)

# 5. High dimension, medium batch
results$high_medium <- profile_operation(
  "High dim (100), medium batch (10000)",
  function() sobol_points(n = 10000, dim = 100),
  times = 5
)

# 6. Incremental generation
results$incremental <- profile_operation(
  "Incremental generation (10000 points, dim=10)",
  function() {
    gen <- sobol_generator(dim = 10)
    for (i in 1:10000) {
      gen$next()
    }
  },
  times = 5
)

# 7. Batch via generator
results$batch_gen <- profile_operation(
  "Batch via generator (10000 points, dim=10)",
  function() {
    gen <- sobol_generator(dim = 10)
    gen$next_n(10000)
  },
  times = 20
)

# 8. Skip operation
results$skip <- profile_operation(
  "Skip operation (skip to 1000000)",
  function() {
    gen <- sobol_generator(dim = 10)
    gen$skip_to(1000000)
  },
  times = 50
)

# Print summary
cat("\n\n=== PROFILING SUMMARY ===\n\n")
for (name in names(results)) {
  r <- results[[name]]
  cat(sprintf("%-40s: %8.4f sec (avg), %10.2f ops/sec\n",
    r$name, r$avg_time, r$ops_per_sec))
}

# Save profiling results
saveRDS(results, file = "profiling_results.rds")
cat("\nResults saved to: profiling_results.rds\n")

# Example Usage of Sobol Sequence Generator
# ==========================================

library(sobol)

# Example 1: Basic batch generation
# ----------------------------------
cat("Example 1: Batch generation\n")
cat("============================\n\n")

# Generate 10 points in 2 dimensions
points <- sobol_points(n = 10, dim = 2)
cat("Generated 10 points in 2 dimensions:\n")
print(head(points))
cat(sprintf("\nDimensions: %d x %d\n\n", nrow(points), ncol(points)))

# Example 2: Incremental generation with state
# ---------------------------------------------
cat("Example 2: Incremental generation\n")
cat("==================================\n\n")

# Create a generator for 3 dimensions
gen <- sobol_generator(dimensions = 3)
print(gen)
cat("\n")

# Generate points one at a time
cat("First point:\n")
print(sobol_next(gen))

cat("\nSecond point:\n")
print(sobol_next(gen))

cat("\nThird point:\n")
print(sobol_next(gen))

# Check current state
cat(sprintf("\nCurrent index: %g\n", sobol_index(gen)))
cat(sprintf("Dimensions: %d\n\n", sobol_dimensions(gen)))

# Example 3: Batch generation from a generator
# ---------------------------------------------
cat("Example 3: Batch from generator\n")
cat("================================\n\n")

gen2 <- sobol_generator(dimensions = 2)
batch <- sobol_next_n(gen2, n = 5)
cat("Generated 5 points at once:\n")
print(batch)
cat("\n")

# Example 4: Skip functionality
# ------------------------------
cat("Example 4: Skip to specific index\n")
cat("==================================\n\n")

# Create generator and skip to index 100
gen3 <- sobol_generator(dimensions = 2, skip = 100)
cat("Generator starting at index 100:\n")
print(gen3)
cat("\nFirst point (index 100):\n")
print(sobol_next(gen3))

# Skip to another index
sobol_skip_to(gen3, 200)
cat("\nAfter skipping to index 200:\n")
cat(sprintf("Current index: %g\n", sobol_index(gen3)))
cat("Next point (index 200):\n")
print(sobol_next(gen3))
cat("\n")

# Example 5: Summary method
# --------------------------
cat("Example 5: Summary information\n")
cat("==============================\n\n")

gen4 <- sobol_generator(dimensions = 5)
sobol_next_n(gen4, n = 50)
cat("After generating 50 points:\n")
print(summary(gen4))
cat("\n")

# Example 6: Reproducibility
# ---------------------------
cat("Example 6: Reproducibility\n")
cat("==========================\n\n")

# Two generators with the same parameters produce the same sequence
gen_a <- sobol_generator(dimensions = 2)
gen_b <- sobol_generator(dimensions = 2)

points_a <- sobol_next_n(gen_a, n = 5)
points_b <- sobol_next_n(gen_b, n = 5)

cat("Generator A points:\n")
print(points_a)
cat("\nGenerator B points:\n")
print(points_b)
cat("\nAre they identical?", identical(points_a, points_b), "\n\n")

# Example 7: Comparison between batch and incremental
# ----------------------------------------------------
cat("Example 7: Batch vs Incremental\n")
cat("================================\n\n")

# Batch generation
batch_result <- sobol_points(n = 3, dim = 2)
cat("Batch generation (first 3 points):\n")
print(batch_result)

# Incremental generation
gen_inc <- sobol_generator(dimensions = 2)
inc_result <- sobol_next_n(gen_inc, n = 3)
cat("\nIncremental generation (first 3 points):\n")
print(inc_result)

cat("\nAre they identical?", identical(batch_result, inc_result), "\n\n")

cat("All examples completed successfully!\n")

# Benchmark comparison: sobol vs pomp::sobol_design
# This script compares the performance of the sobol package with pomp::sobol_design

library(microbenchmark)

# Try to load pomp if available
pomp_available <- requireNamespace("pomp", quietly = TRUE)

if (!pomp_available) {
  warning("pomp package not available. Install it with: install.packages('pomp')")
  warning("Running benchmarks for sobol package only.")
}

# Load sobol package (assumes it's installed or we're running from source)
if (!requireNamespace("sobol", quietly = TRUE)) {
  stop("sobol package must be installed or sourced")
}

library(sobol)

# Helper function to convert sobol_design-like parameters to sobol_points
sobol_wrapper <- function(lower, upper, nseq) {
  # Generate points in [0,1]^d
  dim <- length(lower)
  points <- sobol::sobol_points(n = nseq, dim = dim)

  # Scale to [lower, upper]
  for (i in seq_len(dim)) {
    points[, i] <- lower[i] + points[, i] * (upper[i] - lower[i])
  }

  colnames(points) <- names(lower)
  points
}

# Benchmark configurations
configs <- list(
  small_2d = list(
    lower = c(a = 0, b = 0),
    upper = c(a = 1, b = 1),
    nseq = 100
  ),
  medium_5d = list(
    lower = c(a = 0, b = 0, c = 0, d = 0, e = 0),
    upper = c(a = 1, b = 1, c = 1, d = 1, e = 1),
    nseq = 1000
  ),
  large_10d = list(
    lower = setNames(rep(0, 10), letters[1:10]),
    upper = setNames(rep(1, 10), letters[1:10]),
    nseq = 10000
  ),
  high_dim_50d = list(
    lower = setNames(rep(0, 50), paste0("x", 1:50)),
    upper = setNames(rep(1, 50), paste0("x", 1:50)),
    nseq = 1000
  ),
  extreme_100d = list(
    lower = setNames(rep(0, 100), paste0("x", 1:100)),
    upper = setNames(rep(1, 100), paste0("x", 1:100)),
    nseq = 500
  )
)

# Run benchmarks
results <- list()

for (config_name in names(configs)) {
  config <- configs[[config_name]]
  cat("\n=== Benchmark:", config_name, "===\n")
  cat("Dimensions:", length(config$lower), "\n")
  cat("Sequences:", config$nseq, "\n\n")

  if (pomp_available) {
    # Compare both implementations
    result <- microbenchmark(
      sobol = sobol_wrapper(config$lower, config$upper, config$nseq),
      pomp = pomp::sobol_design(
        lower = config$lower,
        upper = config$upper,
        nseq = config$nseq
      ),
      times = 20,
      unit = "ms"
    )
  } else {
    # Only benchmark sobol
    result <- microbenchmark(
      sobol = sobol_wrapper(config$lower, config$upper, config$nseq),
      times = 50,
      unit = "ms"
    )
  }

  print(result)
  results[[config_name]] <- result
}

# Summary statistics
cat("\n\n=== SUMMARY ===\n\n")

for (config_name in names(results)) {
  cat("Configuration:", config_name, "\n")
  result <- results[[config_name]]

  if (pomp_available) {
    sobol_times <- result$time[result$expr == "sobol"]
    pomp_times <- result$time[result$expr == "pomp"]

    sobol_median <- median(sobol_times) / 1e6  # Convert to ms
    pomp_median <- median(pomp_times) / 1e6
    speedup <- pomp_median / sobol_median

    cat(sprintf("  sobol median: %.3f ms\n", sobol_median))
    cat(sprintf("  pomp median:  %.3f ms\n", pomp_median))
    cat(sprintf("  Speedup:      %.2fx\n", speedup))
  } else {
    sobol_times <- result$time[result$expr == "sobol"]
    sobol_median <- median(sobol_times) / 1e6
    cat(sprintf("  sobol median: %.3f ms\n", sobol_median))
  }
  cat("\n")
}

# Save results for later analysis
saveRDS(results, file = "benchmark_results.rds")
cat("Results saved to: benchmark_results.rds\n")

#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @useDynLib sobol, .registration = TRUE
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
NULL

# Load the Rcpp module when the package is loaded
.onLoad <- function(libname, pkgname) {
  # The module will be automatically loaded by Rcpp
  # This ensures the SobolGenerator class is available
}

#' Generate a Batch of Sobol Points
#'
#' Efficiently generates a matrix of Sobol sequence points. This is a
#' convenience function that does not maintain state between calls.
#' For incremental generation, use \code{\link{sobol_generator}} instead.
#'
#' @name sobol_points
#' @param n Integer, the number of points to generate. Must be non-negative.
#' @param dim Integer, the number of dimensions for each point.
#'   Must be a positive integer.
#' @param skip Numeric, the number of initial points to skip (default: 0).
#'   This allows generating subsequences of the Sobol sequence.
#'
#' @return A numeric matrix with n rows and dim columns, where each
#'   row represents a point in the Sobol sequence. Values are in [0, 1).
#'   If n = 0, returns a 0 x dim matrix.
#'
#' @details
#' This function is implemented directly in C++ via Rcpp and is more
#' efficient than creating a generator and calling \code{sobol_next_n}
#' when you know in advance how many points you need.
#'
#' The skip parameter allows you to start from any point in the sequence,
#' which is useful for:
#' \itemize{
#'   \item Reproducibility: generating the same subsequence across runs
#'   \item Parallelization: different workers can generate
#'     non-overlapping segments
#'   \item Continuation: extending a previous sequence without
#'     regenerating early points
#' }
#'
#' @examples
#' # Generate 1000 points in 5 dimensions
#' points <- sobol_points(n = 1000, dim = 5)
#' dim(points) # [1] 1000    5
#'
#' # Skip the first 100 points
#' points_skipped <- sobol_points(n = 100, dim = 2, skip = 100)
#'
#' # Empty result
#' empty <- sobol_points(n = 0, dim = 3)
#' dim(empty) # [1] 0 3
#'
#' @seealso \code{\link{sobol_generator}} for incremental generation with state
#'
#' @export
#' @useDynLib sobol, .registration = TRUE
#' @importFrom Rcpp sourceCpp
NULL

# Note: The actual sobol_points function is exported from C++ via Rcpp::export
# in src/rcpp_interface.cpp. This documentation file ensures it appears in
# the R documentation with proper help text.

#' Create a Sobol Sequence Generator
#'
#' Creates an S3 object that wraps an Rcpp Sobol sequence generator for
#' incremental point generation. The generator maintains internal state
#' and allows for efficient generation of quasi-random sequences.
#'
#' @param dimensions Integer, the number of dimensions for the Sobol sequence.
#'   Must be a positive integer.
#' @param skip Numeric, the number of initial points to skip (default: 0).
#'   This allows starting the sequence from any index for reproducibility.
#'
#' @return An S3 object of class "sobol_generator" containing:
#'   \item{generator}{The underlying Rcpp reference class object}
#'   \item{dimensions}{Number of dimensions}
#'   \item{initial_skip}{Initial skip value used at construction}
#'
#' @examples
#' # Create a 3-dimensional Sobol generator
#' gen <- sobol_generator(dimensions = 3)
#'
#' # Generate a single point
#' point <- sobol_next(gen)
#'
#' # Generate multiple points
#' points <- sobol_next_n(gen, n = 100)
#'
#' # Skip to a specific index
#' sobol_skip_to(gen, 1000)
#'
#' # Create a generator starting from index 50
#' gen2 <- sobol_generator(dimensions = 2, skip = 50)
#'
#' @export
#' @importFrom methods new
sobol_generator <- function(dimensions, skip = 0) {
  # Validate inputs
  checkmate::assert_int(dimensions, lower = 1)
  checkmate::assert_int(skip, lower = 0)

  # Convert to integer types for R
  dimensions <- as.integer(dimensions)
  skip <- as.numeric(skip)

  # Load the Rcpp module
  sobol_module <- Rcpp::Module("sobol_generator_module", PACKAGE = "sobol")

  # Create the generator
  generator <- tryCatch(
    new(sobol_module$SobolGenerator, dimensions, skip),
    error = function(e) {
      stop("Failed to create Sobol generator: ", e$message)
    }
  )

  # Create S3 object
  obj <- structure(
    list(
      generator = generator,
      dimensions = dimensions,
      initial_skip = skip
    ),
    class = "sobol_generator"
  )

  obj
}

#' Generate the Next Point from a Sobol Generator
#'
#' Generates a single point from the Sobol sequence and advances the
#' internal state of the generator.
#'
#' @param x A sobol_generator object created by \code{\link{sobol_generator}}
#' @param ... Additional arguments (currently unused)
#'
#' @return A numeric vector of length equal to the number of dimensions,
#'   containing the next point in the Sobol sequence. Values are in [0, 1).
#'
#' @examples
#' gen <- sobol_generator(dimensions = 3)
#' point <- sobol_next(gen)
#' print(point) # e.g., [0.5, 0.5, 0.5]
#'
#' @export
sobol_next <- function(x, ...) {
  checkmate::assert_class(x, "sobol_generator")

  tryCatch(
    x$generator$`next`(),
    error = function(e) {
      stop("Failed to generate next point: ", e$message)
    }
  )
}

#' Generate Multiple Points from a Sobol Generator
#'
#' Generates n consecutive points from the Sobol sequence and advances the
#' internal state of the generator.
#'
#' @param x A sobol_generator object created by \code{\link{sobol_generator}}
#' @param n Integer, the number of points to generate. Must be non-negative.
#' @param ... Additional arguments (currently unused)
#'
#' @return A numeric matrix with n rows and dimensions columns, where each
#'   row represents a point in the Sobol sequence. Values are in [0, 1).
#'   If n = 0, returns a 0 x dimensions matrix.
#'
#' @examples
#' gen <- sobol_generator(dimensions = 2)
#' points <- sobol_next_n(gen, n = 10)
#' print(dim(points)) # [1] 10  2
#'
#' @export
sobol_next_n <- function(x, n, ...) {
  checkmate::assert_class(x, "sobol_generator")
  checkmate::assert_int(n, lower = 0)

  n <- as.integer(n)

  tryCatch(
    x$generator$next_n(n),
    error = function(e) {
      stop("Failed to generate points: ", e$message)
    }
  )
}

#' Skip to a Specific Index in a Sobol Generator
#'
#' Jumps the internal state of the generator to a specific index in the
#' Sobol sequence. This allows for reproducible subsequences and parallel
#' generation strategies.
#'
#' @param x A sobol_generator object created by \code{\link{sobol_generator}}
#' @param index Numeric, the index to skip to. Must be a non-negative integer.
#'   The next call to \code{sobol_next} will return the point at this index.
#' @param ... Additional arguments (currently unused)
#'
#' @return Invisibly returns the sobol_generator object (for chaining).
#'   The primary purpose is the side effect of updating the internal state.
#'
#' @examples
#' gen <- sobol_generator(dimensions = 2)
#' sobol_skip_to(gen, 100)
#' point <- sobol_next(gen) # This is the 100th point (0-indexed)
#'
#' @export
sobol_skip_to <- function(x, index, ...) {
  checkmate::assert_class(x, "sobol_generator")
  checkmate::assert_number(index, lower = 0, finite = TRUE)

  index <- as.numeric(index)

  tryCatch(
    x$generator$skip_to(index),
    error = function(e) {
      stop("Failed to skip to index: ", e$message)
    }
  )

  # Return the object invisibly for chaining (CRAN compliance - not just NULL)
  invisible(x)
}

#' Get Current Index of a Sobol Generator
#'
#' Returns the current index of the generator, which is the index of the
#' next point that will be generated.
#'
#' @param x A sobol_generator object created by \code{\link{sobol_generator}}
#' @param ... Additional arguments (currently unused)
#'
#' @return A numeric value representing the current index (0-based).
#'
#' @examples
#' gen <- sobol_generator(dimensions = 2)
#' sobol_next(gen)
#' sobol_next(gen)
#' idx <- sobol_index(gen)
#' print(idx) # 2
#'
#' @export
sobol_index <- function(x, ...) {
  checkmate::assert_class(x, "sobol_generator")

  tryCatch(
    x$generator$index(),
    error = function(e) {
      stop("Failed to get index: ", e$message)
    }
  )
}

#' Get Number of Dimensions of a Sobol Generator
#'
#' Returns the number of dimensions configured for the generator.
#'
#' @param x A sobol_generator object created by \code{\link{sobol_generator}}
#' @param ... Additional arguments (currently unused)
#'
#' @return An integer representing the number of dimensions.
#'
#' @examples
#' gen <- sobol_generator(dimensions = 5)
#' dims <- sobol_dimensions(gen)
#' print(dims) # 5
#'
#' @export
sobol_dimensions <- function(x, ...) {
  checkmate::assert_class(x, "sobol_generator")

  tryCatch(
    x$generator$dimensions(),
    error = function(e) {
      stop("Failed to get dimensions: ", e$message)
    }
  )
}

#' Print Method for Sobol Generator
#'
#' @param x A sobol_generator object
#' @param ... Additional arguments passed to print
#'
#' @return Invisibly returns the input object (for chaining)
#'
#' @export
print.sobol_generator <- function(x, ...) {
  cat("<Sobol Sequence Generator>\n")
  cat(sprintf("  Dimensions: %d\n", x$dimensions))
  cat(sprintf("  Initial skip: %g\n", x$initial_skip))

  # Get current index
  current_idx <- tryCatch(
    x$generator$index(),
    error = function(e) NA
  )

  if (!is.na(current_idx)) {
    cat(sprintf("  Current index: %g\n", current_idx))
    cat(sprintf("  Points generated: %g\n", current_idx - x$initial_skip))
  }

  invisible(x)
}

#' Summary Method for Sobol Generator
#'
#' @param object A sobol_generator object
#' @param ... Additional arguments passed to summary
#'
#' @return A list with class "summary.sobol_generator" containing summary
#' information
#'
#' @export
summary.sobol_generator <- function(object, ...) {
  current_idx <- tryCatch(
    object$generator$index(),
    error = function(e) NA
  )

  info <- list(
    dimensions = object$dimensions,
    initial_skip = object$initial_skip,
    current_index = current_idx,
    points_generated = if (!is.na(current_idx))
      current_idx - object$initial_skip else NA
  )

  class(info) <- "summary.sobol_generator"
  return(info)
}

#' Print Summary of Sobol Generator
#'
#' @param x A summary.sobol_generator object
#' @param ... Additional arguments passed to print
#'
#' @return Invisibly returns the input object
#'
#' @export
print.summary.sobol_generator <- function(x, ...) {
  cat("Sobol Sequence Generator Summary\n")
  cat("================================\n")
  cat(sprintf("Dimensions:        %d\n", x$dimensions))
  cat(sprintf("Initial skip:      %g\n", x$initial_skip))

  if (!is.na(x$current_index)) {
    cat(sprintf("Current index:     %g\n", x$current_index))
    cat(sprintf("Points generated:  %g\n", x$points_generated))
  } else {
    cat("Current index:     <unavailable>\n")
  }

  invisible(x)
}

# Test S3 Constructor and Object Creation

test_that("sobol_generator creates valid S3 object", {
  gen <- sobol_generator(dimensions = 3)

  expect_s3_class(gen, "sobol_generator")
  expect_type(gen, "list")
  expect_named(gen, c("generator", "dimensions", "initial_skip"))
  expect_equal(gen$dimensions, 3L)
  expect_equal(gen$initial_skip, 0)
})

test_that("sobol_generator accepts skip parameter", {
  gen <- sobol_generator(dimensions = 2, skip = 100)

  expect_s3_class(gen, "sobol_generator")
  expect_equal(gen$initial_skip, 100)
  expect_equal(sobol_index(gen), 100)
})

test_that("sobol_generator validates dimensions", {
  expect_error(sobol_generator(dimensions = 0), "dimensions")
  expect_error(sobol_generator(dimensions = -1), "dimensions")
  expect_error(sobol_generator(dimensions = 1.5), "dimensions")
  expect_error(sobol_generator(dimensions = "3"), "dimensions")
})

test_that("sobol_generator validates skip", {
  expect_error(sobol_generator(dimensions = 2, skip = -1), "skip")
  expect_error(sobol_generator(dimensions = 2, skip = 1.5), "skip")
  expect_error(sobol_generator(dimensions = 2, skip = "10"), "skip")
})

# Test Point Generation

test_that("sobol_next generates single point", {
  gen <- sobol_generator(dimensions = 3)
  point <- sobol_next(gen)

  expect_type(point, "double")
  expect_length(point, 3)
  expect_true(all(point >= 0 & point < 1))
})

test_that("sobol_next advances state", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 1)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 2)
})

test_that("sobol_next validates input", {
  gen <- sobol_generator(dimensions = 2)
  expect_error(sobol_next("not a generator"), "sobol_generator")
  expect_error(sobol_next(list(a = 1)), "sobol_generator")
})

test_that("sobol_next_n generates multiple points", {
  gen <- sobol_generator(dimensions = 2)
  points <- sobol_next_n(gen, n = 10)

  expect_type(points, "double")
  expect_true(is.matrix(points))
  expect_equal(dim(points), c(10, 2))
  expect_true(all(points >= 0 & points < 1))
})

test_that("sobol_next_n handles n=0", {
  gen <- sobol_generator(dimensions = 3)
  points <- sobol_next_n(gen, n = 0)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(0, 3))
})

test_that("sobol_next_n advances state correctly", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_next_n(gen, n = 5)
  expect_equal(sobol_index(gen), 5)
  sobol_next_n(gen, n = 3)
  expect_equal(sobol_index(gen), 8)
})

test_that("sobol_next_n validates inputs", {
  gen <- sobol_generator(dimensions = 2)

  expect_error(sobol_next_n("not a generator", 5), "sobol_generator")
  expect_error(sobol_next_n(gen, n = -1), "n")
  expect_error(sobol_next_n(gen, n = 1.5), "n")
  expect_error(sobol_next_n(gen, n = "5"), "n")
})

# Test Skip Functionality

test_that("sobol_skip_to changes state", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_skip_to(gen, 100)
  expect_equal(sobol_index(gen), 100)
  sobol_skip_to(gen, 50)
  expect_equal(sobol_index(gen), 50)
})

test_that("sobol_skip_to returns object invisibly", {
  gen <- sobol_generator(dimensions = 2)
  result <- withVisible(sobol_skip_to(gen, 10))

  expect_false(result$visible)
  expect_identical(result$value, gen)
})

test_that("sobol_skip_to validates inputs", {
  gen <- sobol_generator(dimensions = 2)

  expect_error(sobol_skip_to("not a generator", 10), "sobol_generator")
  expect_error(sobol_skip_to(gen, -1), "index")
  expect_error(sobol_skip_to(gen, 1.5), "index")
  expect_error(sobol_skip_to(gen, "10"), "index")
})

# Test Query Functions

test_that("sobol_index returns current index", {
  gen <- sobol_generator(dimensions = 2, skip = 50)

  expect_equal(sobol_index(gen), 50)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 51)
})

test_that("sobol_dimensions returns dimensions", {
  gen1 <- sobol_generator(dimensions = 2)
  gen2 <- sobol_generator(dimensions = 5)

  expect_equal(sobol_dimensions(gen1), 2L)
  expect_equal(sobol_dimensions(gen2), 5L)
})

test_that("query functions validate input", {
  expect_error(sobol_index("not a generator"), "sobol_generator")
  expect_error(sobol_dimensions("not a generator"), "sobol_generator")
})

# Test Batch Function

test_that("sobol_points generates correct dimensions", {
  points <- sobol_points(n = 10, dim = 3)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(10, 3))
  expect_true(all(points >= 0 & points < 1))
})

test_that("sobol_points handles skip parameter", {
  # Points without skip (first 5)
  points1 <- sobol_points(n = 5, dim = 2, skip = 0)

  # Points with skip (skip first 5, get next 5)
  points2 <- sobol_points(n = 5, dim = 2, skip = 5)

  # They should be different
  expect_false(identical(points1, points2))
})

test_that("sobol_points handles n=0", {
  points <- sobol_points(n = 0, dim = 3)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(0, 3))
})

# Test S3 Methods

test_that("print.sobol_generator works", {
  gen <- sobol_generator(dimensions = 3, skip = 10)

  expect_output(print(gen), "Sobol Sequence Generator")
  expect_output(print(gen), "Dimensions: 3")
  expect_output(print(gen), "Initial skip: 10")
  expect_output(print(gen), "Current index: 10")

  # Print returns invisibly
  result <- withVisible(print(gen))
  expect_false(result$visible)
  expect_identical(result$value, gen)
})

test_that("summary.sobol_generator creates summary object", {
  gen <- sobol_generator(dimensions = 4)
  sobol_next_n(gen, n = 20)

  summ <- summary(gen)

  expect_s3_class(summ, "summary.sobol_generator")
  expect_type(summ, "list")
  expect_named(summ, c("dimensions", "initial_skip", "current_index", "points_generated"))
  expect_equal(summ$dimensions, 4L)
  expect_equal(summ$initial_skip, 0)
  expect_equal(summ$current_index, 20)
  expect_equal(summ$points_generated, 20)
})

test_that("print.summary.sobol_generator works", {
  gen <- sobol_generator(dimensions = 2, skip = 5)
  sobol_next_n(gen, n = 10)
  summ <- summary(gen)

  expect_output(print(summ), "Sobol Sequence Generator Summary")
  expect_output(print(summ), "Dimensions:\\s+2")
  expect_output(print(summ), "Initial skip:\\s+5")
  expect_output(print(summ), "Current index:\\s+15")
  expect_output(print(summ), "Points generated:\\s+10")

  # Print returns invisibly
  result <- withVisible(print(summ))
  expect_false(result$visible)
  expect_identical(result$value, summ)
})

# Test Reproducibility

test_that("generators with same parameters produce same sequence", {
  gen1 <- sobol_generator(dimensions = 3)
  gen2 <- sobol_generator(dimensions = 3)

  points1 <- sobol_next_n(gen1, n = 10)
  points2 <- sobol_next_n(gen2, n = 10)

  expect_identical(points1, points2)
})

test_that("skip produces consistent results", {
  # Generate first 10, then next 5
  gen1 <- sobol_generator(dimensions = 2)
  sobol_next_n(gen1, n = 10)
  points1 <- sobol_next_n(gen1, n = 5)

  # Generate with skip to get points 10-14
  gen2 <- sobol_generator(dimensions = 2, skip = 10)
  points2 <- sobol_next_n(gen2, n = 5)

  expect_identical(points1, points2)
})

test_that("batch and incremental produce same results", {
  # Batch generation
  batch <- sobol_points(n = 10, dim = 3)

  # Incremental generation
  gen <- sobol_generator(dimensions = 3)
  incremental <- sobol_next_n(gen, n = 10)

  expect_identical(batch, incremental)
})

# Test Edge Cases

test_that("handles large dimensions", {
  gen <- sobol_generator(dimensions = 100)

  point <- sobol_next(gen)
  expect_length(point, 100)
  expect_true(all(point >= 0 & point < 1))
})

test_that("handles large skip values", {
  # Test with a large skip value
  gen <- sobol_generator(dimensions = 2, skip = 1e6)

  expect_equal(sobol_index(gen), 1e6)
  point <- sobol_next(gen)
  expect_length(point, 2)
  expect_equal(sobol_index(gen), 1e6 + 1)
})

test_that("sequence properties hold", {
  # First point should be [0, 0, ...] (index 0)
  gen <- sobol_generator(dimensions = 3)
  first_point <- sobol_next(gen)

  # The first Sobol point at index 0 is (0, 0, 0, ...)
  expect_equal(first_point[1], 0)
  expect_equal(first_point[2], 0)
  expect_equal(first_point[3], 0)

  # Second point at index 1 should be [0.5, 0.5, 0.5, ...]
  second_point <- sobol_next(gen)
  expect_equal(second_point[1], 0.5)
})

# Enhanced Comprehensive Tests for Sobol R Package
# Tests for API usability, edge cases, output validation, and comparison with C++ backend

# Test S3 Constructor and Object Creation
test_that("sobol_generator creates valid S3 object", {
  gen <- sobol_generator(dimensions = 3)

  expect_s3_class(gen, "sobol_generator")
  expect_type(gen, "list")
  expect_named(gen, c("generator", "dimensions", "initial_skip"))
  expect_equal(gen$dimensions, 3L)
  expect_equal(gen$initial_skip, 0)
})

test_that("sobol_generator accepts skip parameter", {
  gen <- sobol_generator(dimensions = 2, skip = 100)

  expect_s3_class(gen, "sobol_generator")
  expect_equal(gen$initial_skip, 100)
  expect_equal(sobol_index(gen), 100)
})

test_that("sobol_generator validates dimensions", {
  expect_error(sobol_generator(dimensions = 0), "Assertion on 'dimensions'")
  expect_error(sobol_generator(dimensions = -1), "Assertion on 'dimensions'")
  expect_error(sobol_generator(dimensions = 1.5), "Assertion on 'dimensions'")
  expect_error(sobol_generator(dimensions = "3"), "Assertion on 'dimensions'")
})

test_that("sobol_generator validates skip", {
  expect_error(sobol_generator(dimensions = 2, skip = -1), "Assertion on 'skip' failed")
  expect_error(sobol_generator(dimensions = 2, skip = 1.5), "Assertion on 'skip' failed")
  expect_error(sobol_generator(dimensions = 2, skip = "10"), "Assertion on 'skip' failed")
})

# Test Point Generation
test_that("sobol_next generates single point", {
  gen <- sobol_generator(dimensions = 3)
  point <- sobol_next(gen)

  expect_type(point, "double")
  expect_length(point, 3)
  expect_true(all(point >= 0 & point < 1))
})

test_that("sobol_next advances state", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 1)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 2)
})

test_that("sobol_next validates input", {
  gen <- sobol_generator(dimensions = 2)
  expect_error(sobol_next("not a generator"), "Assertion on 'x'")
  expect_error(sobol_next(list(a = 1)), "Assertion on 'x' failed: Must inherit from class 'sobol_generator',")
})

test_that("sobol_next_n generates multiple points", {
  gen <- sobol_generator(dimensions = 2)
  points <- sobol_next_n(gen, n = 10)

  expect_type(points, "double")
  expect_true(is.matrix(points))
  expect_equal(dim(points), c(10, 2))
  expect_true(all(points >= 0 & points < 1))
})

test_that("sobol_next_n handles n=0", {
  gen <- sobol_generator(dimensions = 3)
  points <- sobol_next_n(gen, n = 0)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(0, 3))
})

test_that("sobol_next_n advances state correctly", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_next_n(gen, n = 5)
  expect_equal(sobol_index(gen), 5)
  sobol_next_n(gen, n = 3)
  expect_equal(sobol_index(gen), 8)
})

test_that("sobol_next_n validates inputs", {
  gen <- sobol_generator(dimensions = 2)

  expect_error(sobol_next_n("not a generator", 5), "Assertion on 'x' failed: Must inherit from class 'sobol_generator',")
  expect_error(sobol_next_n(gen, n = -1), "Assertion on 'n' failed: Element 1 is not >= 0.")
  expect_error(sobol_next_n(gen, n = 1.5), "Assertion on 'n' failed: Must be of type 'single integerish value', not 'double'.")
  expect_error(sobol_next_n(gen, n = "5"), "Assertion on 'n' failed: Must be of type 'single integerish value', not 'character'.")
})

# Test Skip Functionality
test_that("sobol_skip_to changes state", {
  gen <- sobol_generator(dimensions = 2)

  expect_equal(sobol_index(gen), 0)
  sobol_skip_to(gen, 100)
  expect_equal(sobol_index(gen), 100)
  sobol_skip_to(gen, 50)
  expect_equal(sobol_index(gen), 50)
})

test_that("sobol_skip_to returns object invisibly", {
  gen <- sobol_generator(dimensions = 2)
  result <- withVisible(sobol_skip_to(gen, 10))

  expect_false(result$visible)
  expect_identical(result$value, gen)
})

test_that("sobol_skip_to validates inputs", {
  gen <- sobol_generator(dimensions = 2)

  expect_error(sobol_skip_to("not a generator", 10), "Assertion on 'x' failed: Must inherit from class 'sobol_generator', but has class 'character'.")
  expect_error(sobol_skip_to(gen, -1), "Assertion on 'index' failed: Element 1 is not >= 0.")
  expect_error(sobol_skip_to(gen, 1.5), "Failed to skip to index: SobolGenerator::skip_to failed: point_index must be an integer value")
  expect_error(sobol_skip_to(gen, "10"), "Assertion on 'index' failed: Must be of type 'number', not 'character'.")
})

# Test Query Functions
test_that("sobol_index returns current index", {
  gen <- sobol_generator(dimensions = 2, skip = 50)

  expect_equal(sobol_index(gen), 50)
  sobol_next(gen)
  expect_equal(sobol_index(gen), 51)
})

test_that("sobol_dimensions returns dimensions", {
  gen1 <- sobol_generator(dimensions = 2)
  gen2 <- sobol_generator(dimensions = 5)

  expect_equal(sobol_dimensions(gen1), 2L)
  expect_equal(sobol_dimensions(gen2), 5L)
})

test_that("query functions validate input", {
  expect_error(sobol_index("not a generator"), "Assertion on 'x' failed: Must inherit from class 'sobol_generator', but has class 'character'.")
  expect_error(sobol_dimensions("not a generator"), "Assertion on 'x' failed: Must inherit from class 'sobol_generator', but has class 'character'.")
})

# Test Batch Function
test_that("sobol_points generates correct dimensions", {
  points <- sobol_points(n = 10, dim = 3)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(10, 3))
  expect_true(all(points >= 0 & points < 1))
})

test_that("sobol_points handles skip parameter", {
  # Points without skip (first 5)
  points1 <- sobol_points(n = 5, dim = 2, skip = 0)

  # Points with skip (skip first 5, get next 5)
  points2 <- sobol_points(n = 5, dim = 2, skip = 5)

  # They should be different
  expect_false(identical(points1, points2))
})

test_that("sobol_points handles n=0", {
  points <- sobol_points(n = 0, dim = 3)

  expect_true(is.matrix(points))
  expect_equal(dim(points), c(0, 3))
})

# Test S3 Methods
test_that("print.sobol_generator works", {
  gen <- sobol_generator(dimensions = 3, skip = 10)

  expect_output(print(gen), "Sobol Sequence Generator")
  expect_output(print(gen), "Dimensions: 3")
  expect_output(print(gen), "Initial skip: 10")
  expect_output(print(gen), "Current index: 10")

  # Print returns invisibly
  result <- withVisible(print(gen))
  expect_false(result$visible)
  expect_identical(result$value, gen)
})

test_that("summary.sobol_generator creates summary object", {
  gen <- sobol_generator(dimensions = 4)
  sobol_next_n(gen, n = 20)

  summ <- summary(gen)

  expect_s3_class(summ, "summary.sobol_generator")
  expect_type(summ, "list")
  expect_named(summ, c("dimensions", "initial_skip", "current_index", "points_generated"))
  expect_equal(summ$dimensions, 4L)
  expect_equal(summ$initial_skip, 0)
  expect_equal(summ$current_index, 20)
  expect_equal(summ$points_generated, 20)
})

test_that("print.summary.sobol_generator works", {
  gen <- sobol_generator(dimensions = 2, skip = 5)
  sobol_next_n(gen, n = 10)
  summ <- summary(gen)

  expect_output(print(summ), "Sobol Sequence Generator Summary")
  expect_output(print(summ), "Dimensions:\\s+2")
  expect_output(print(summ), "Initial skip:\\s+5")
  expect_output(print(summ), "Current index:\\s+15")
  expect_output(print(summ), "Points generated:\\s+10")

  # Print returns invisibly
  result <- withVisible(print(summ))
  expect_false(result$visible)
  expect_identical(result$value, summ)
})

# Test Reproducibility
test_that("generators with same parameters produce same sequence", {
  gen1 <- sobol_generator(dimensions = 3)
  gen2 <- sobol_generator(dimensions = 3)

  points1 <- sobol_next_n(gen1, n = 10)
  points2 <- sobol_next_n(gen2, n = 10)

  expect_identical(points1, points2)
})

test_that("skip produces consistent results", {
  # Generate first 10, then next 5
  gen1 <- sobol_generator(dimensions = 2)
  sobol_next_n(gen1, n = 10)
  points1 <- sobol_next_n(gen1, n = 5)

  # Generate with skip to get points 10-14
  gen2 <- sobol_generator(dimensions = 2, skip = 10)
  points2 <- sobol_next_n(gen2, n = 5)

  expect_identical(points1, points2)
})

test_that("batch and incremental produce same results", {
  # Batch generation
  batch <- sobol_points(n = 10, dim = 3)

  # Incremental generation
  gen <- sobol_generator(dimensions = 3)
  incremental <- sobol_next_n(gen, n = 10)

  expect_identical(batch, incremental)
})

# Test Edge Cases
test_that("handles large dimensions", {
  gen <- sobol_generator(dimensions = 100)

  point <- sobol_next(gen)
  expect_length(point, 100)
  expect_true(all(point >= 0 & point < 1))
})

test_that("handles very large dimensions (1000)", {
  gen <- sobol_generator(dimensions = 1000)

  point <- sobol_next(gen)
  expect_length(point, 1000)
  expect_true(all(point >= 0 & point < 1))
})

test_that("handles large skip values", {
  # Test with a large skip value
  gen <- sobol_generator(dimensions = 2, skip = 1e6)

  expect_equal(sobol_index(gen), 1e6)
  point <- sobol_next(gen)
  expect_length(point, 2)
  expect_equal(sobol_index(gen), 1e6 + 1)
})

test_that("handles very large skip values", {
  # Test with an even larger skip value (10 million)
  gen <- sobol_generator(dimensions = 3, skip = 1e7)

  expect_equal(sobol_index(gen), 1e7)
  point <- sobol_next(gen)
  expect_length(point, 3)
  expect_true(all(point >= 0 & point < 1))
})

# Test Mathematical Correctness with Reference Values
test_that("1D sequence matches reference values", {
  gen <- sobol_generator(dimensions = 1)

  expected <- c(0.0, 0.5, 0.75, 0.25, 0.375, 0.875, 0.625, 0.125)

  for (i in seq_along(expected)) {
    point <- sobol_next(gen)
    expect_equal(point[1], expected[i], tolerance = 1e-10)
  }
})

test_that("2D sequence matches reference values", {
  gen <- sobol_generator(dimensions = 2)

  expected <- matrix(c(
    0.0, 0.0,
    0.5, 0.5,
    0.75, 0.25,
    0.25, 0.75
  ), ncol = 2, byrow = TRUE)

  for (i in seq_len(nrow(expected))) {
    point <- sobol_next(gen)
    expect_equal(point, expected[i, ], tolerance = 1e-10)
  }
})

test_that("3D sequence matches reference values", {
  gen <- sobol_generator(dimensions = 3)

  expected <- matrix(c(
    0.0, 0.0, 0.0,
    0.5, 0.5, 0.5,
    0.75, 0.25, 0.25,
    0.25, 0.75, 0.75,
    0.375, 0.625, 0.125,
    0.875, 0.125, 0.625,
    0.625, 0.875, 0.375,
    0.125, 0.375, 0.875
  ), ncol = 3, byrow = TRUE)

  for (i in 1:nrow(expected)) {
    point <- sobol_next(gen)
    expect_equal(point, expected[i, ], tolerance = 1e-10)
  }
})

# Test Sequence Properties
test_that("first point is all zeros", {
  for (dim in c(1, 2, 3, 5, 10)) {
    gen <- sobol_generator(dimensions = dim)
    first_point <- sobol_next(gen)
    expect_equal(first_point, rep(0, dim), tolerance = 1e-10)
  }
})

test_that("second point is all 0.5s", {
  for (dim in c(1, 2, 3, 5, 10)) {
    gen <- sobol_generator(dimensions = dim)
    sobol_next(gen) # Skip first point
    second_point <- sobol_next(gen)
    expect_equal(second_point, rep(0.5, dim), tolerance = 1e-10)
  }
})

test_that("all points are in [0, 1) range", {
  gen <- sobol_generator(dimensions = 5)

  for (i in 1:1000) {
    point <- sobol_next(gen)
    expect_true(all(point >= 0))
    expect_true(all(point < 1))
  }
})

test_that("sequence has low discrepancy", {
  # Generate many points and check distribution
  gen <- sobol_generator(dimensions = 2)
  points <- sobol_next_n(gen, n = 1024)

  # Check each dimension separately
  for (d in 1:2) {
    dim_points <- points[, d]

    # Split into 8 bins and check distribution
    bins <- cut(dim_points, breaks = seq(0, 1, length.out = 9), include.lowest = TRUE)
    bin_counts <- table(bins)

    # Each bin should have approximately 1024/8 = 128 points
    # Allow for variation but should be reasonably uniform
    expect_true(all(bin_counts > 64))
    expect_true(all(bin_counts < 192))
  }
})

# Test Consistency with C++ Backend
test_that("R matches C++ for multiple dimensions", {
  for (dim in c(1, 2, 5, 10, 50, 100)) {
    gen <- sobol_generator(dimensions = dim)
    points_r <- sobol_next_n(gen, n = 20)

    # Generate again from scratch
    points_batch <- sobol_points(n = 20, dim = dim)

    expect_identical(points_r, points_batch)
  }
})

test_that("skip_to produces correct results", {
  # Generate using skip_to
  gen1 <- sobol_generator(dimensions = 3)
  sobol_skip_to(gen1, 50)
  point1 <- sobol_next(gen1)

  # Generate by advancing incrementally
  gen2 <- sobol_generator(dimensions = 3)
  for (i in 1:50) {
    sobol_next(gen2)
  }
  point2 <- sobol_next(gen2)

  expect_equal(point1, point2, tolerance = 1e-10)
})

# Test API Usability
test_that("generator can be reused after skip_to", {
  gen <- sobol_generator(dimensions = 2)

  # Generate some points
  points1 <- sobol_next_n(gen, n = 5)

  # Skip back
  sobol_skip_to(gen, 0)

  # Generate again
  points2 <- sobol_next_n(gen, n = 5)

  expect_identical(points1, points2)
})

test_that("multiple generators are independent", {
  gen1 <- sobol_generator(dimensions = 2)
  gen2 <- sobol_generator(dimensions = 2)

  # Advance gen1
  sobol_next_n(gen1, n = 10)

  # gen2 should still be at index 0
  expect_equal(sobol_index(gen2), 0)

  # gen1 and gen2 should produce different next points
  point1 <- sobol_next(gen1)
  point2 <- sobol_next(gen2)

  expect_false(identical(point1, point2))
})

# Test Error Handling
test_that("handles invalid dimensions gracefully", {
  expect_error(sobol_generator(dimensions = 0))
  expect_error(sobol_generator(dimensions = -5))
  expect_error(sobol_generator(dimensions = NA))
  expect_error(sobol_generator(dimensions = NULL))
})

test_that("handles invalid skip gracefully", {
  expect_error(sobol_generator(dimensions = 2, skip = -1))
  expect_error(sobol_generator(dimensions = 2, skip = NA))
  expect_error(sobol_generator(dimensions = 2, skip = NULL))
})

test_that("sobol_points validates parameters", {
  expect_error(sobol_points(n = -1, dim = 2), "non-negative")
  expect_error(sobol_points(n = 10, dim = 0), "greater than zero")
  # Note: C++ implementation may not validate that n is an integer
  # This is acceptable as fractional n will be truncated to integer
})

# Performance and Scalability Tests
test_that("can generate large batches efficiently", {
  gen <- sobol_generator(dimensions = 10)

  # This should complete without error or timeout
  points <- sobol_next_n(gen, n = 10000)

  expect_equal(nrow(points), 10000)
  expect_equal(ncol(points), 10)
})

test_that("batch generation matches incremental for large n", {
  dim <- 5
  n <- 1000

  # Batch generation
  batch <- sobol_points(n = n, dim = dim)

  # Incremental generation
  gen <- sobol_generator(dimensions = dim)
  incremental <- sobol_next_n(gen, n = n)

  expect_identical(batch, incremental)
})

# Additional Coverage Tests
test_that("column names are preserved in matrix output", {
  gen <- sobol_generator(dimensions = 3)
  points <- sobol_next_n(gen, n = 5)

  # Matrix should have proper dimensions
  expect_equal(ncol(points), 3)
  expect_equal(nrow(points), 5)
})

test_that("works with single dimension", {
  gen <- sobol_generator(dimensions = 1)

  points <- sobol_next_n(gen, n = 10)
  expect_equal(dim(points), c(10, 1))
  expect_true(all(points >= 0 & points < 1))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sobol_points <- function(n, dim, skip = 0.0) {
    .Call(`_sobol_sobol_points`, n, dim, skip)
}

#' Generate a Sobol Design for Parameter Exploration
#'
#' Creates a Latin hypercube design based on the Sobol low-discrepancy sequence.
#' This function provides an API-compatible alternative to the sobol_design
#' function in the pomp-explore package for generating parameter designs.
#'
#' @param lower Named numeric vector giving the lower bounds of the parameter
#' ranges. Must have the same names as \code{upper}.
#' @param upper Named numeric vector giving the upper bounds of the parameter
#' ranges. Must have the same names as \code{lower}.
#' @param nseq Integer, the total number of parameter sets (points) to generate.
#'
#' @return A data frame with \code{nseq} rows and one column for each parameter
#'   named in \code{lower} and \code{upper}. Each column contains values scaled
#'   to the specified range [lower, upper] for that parameter.
#'
#' @details
#' This function generates a Sobol sequence in the unit hypercube [0,1]^d and
#' then scales each dimension to the specified parameter ranges. The Sobol
#' sequence is generated using the Joe-Kuo direction numbers with Property A
#' enforcement, providing excellent low-discrepancy properties.
#'
#' Following the recommendation of Joe & Kuo (2003) and the implementation
#' in pomp-explore, this function skips the first k points of the Sobol
#' sequence, where k is the largest power of 2 smaller than nseq. This
#' improves the uniformity properties of the generated design.
#'
#' The function is designed to be API-compatible with the \code{sobol_design}
#' function from the pomp-explore package, allowing for easy comparison and
#' drop-in replacement.
#'
#' @examples
#' # Generate 100 parameter sets for two parameters
#' design <- sobol_design(
#'   lower = c(a = 0, b = 100),
#'   upper = c(a = 1, b = 200),
#'   nseq = 100
#' )
#' head(design)
#'
#' # Plot the design
#' plot(design$a, design$b, main = "Sobol Design")
#'
#' # High-dimensional example
#' params <- paste0("param", 1:10)
#' design_10d <- sobol_design(
#'   lower = setNames(rep(0, 10), params),
#'   upper = setNames(rep(1, 10), params),
#'   nseq = 1000
#' )
#'
#' @references
#' Bratley, P., & Fox, B. L. (1988). Algorithm 659: Implementing Sobol's
#' quasirandom sequence generator. ACM Transactions on Mathematical Software,
#' 14(1), 88-100.
#'
#' Joe, S., & Kuo, F. Y. (2008). Constructing Sobol sequences with better
#' two-dimensional projections. SIAM Journal on Scientific Computing, 30(5),
#' 2635-2654.
#'
#' @seealso \code{\link{sobol_points}} for batch generation without scaling,
#'   \code{\link{sobol_generator}} for incremental generation
#'
#' @export
sobol_design <- function(lower = numeric(0), upper = numeric(0), nseq) {
  # Validate inputs
  checkmate::assert_numeric(lower, finite = TRUE, any.missing = FALSE,
                            min.len = 1, names = "named")
  checkmate::assert_numeric(upper, finite = TRUE, any.missing = FALSE,
                            min.len = 1, names = "named")
  checkmate::assert_true(length(lower) == length(upper))
  checkmate::assert_set_equal(names(lower), names(upper))
  checkmate::assert_int(nseq, lower = 1, upper = 1073741824L)

  # Reorder upper to match lower's names before validation
  upper <- upper[names(lower)]

  # Check that lower < upper for all elements
  checkmate::assert_true(
    all(lower < upper),
    .var.name = "all elements of 'lower' must be less than \
    corresponding elements of 'upper'"
  )

  # Get number of dimensions
  d <- length(lower)

  # Calculate skip count following Joe & Kuo (2003) recommendation:
  # Skip the largest power of 2 smaller than nseq for better uniformity
  skip_count <- 1L
  while (skip_count * 2L < nseq) {
    skip_count <- skip_count * 2L
  }

  # Generate Sobol points in [0, 1]^d
  points <- tryCatch(
    sobol_points(
      n = as.integer(nseq),
      dim = as.integer(d),
      skip = as.numeric(skip_count)
    ),
    error = function(e) {
      stop("Failed to generate Sobol sequence: ", e$message)
    }
  )

  # Scale each dimension to [lower, upper]
  scaled <- vapply(
    seq_len(d),
    function(k) {
      lower[k] + (upper[k] - lower[k]) * points[, k]
    },
    numeric(nseq)
  )

  # Ensure result is always a matrix (vapply returns a vector when nseq == 1)
  if (!is.matrix(scaled)) {
    scaled <- matrix(scaled, nrow = nseq, ncol = d)
  }

  # Convert to data frame with proper column names
  lnames <- names(lower)
  colnames(scaled) <- lnames
  as.data.frame(scaled)
}

library(testthat)
test_that("sobol_design basic functionality works", {
  # Basic 2D design
  design <- sobol_design(
    lower = c(a = 0, b = 0),
    upper = c(a = 1, b = 1),
    nseq = 100
  )

  expect_s3_class(design, "data.frame")
  expect_equal(nrow(design), 100)
  expect_equal(ncol(design), 2)
  expect_equal(names(design), c("a", "b"))
  expect_true(all(design$a >= 0 & design$a <= 1))
  expect_true(all(design$b >= 0 & design$b <= 1))
})

test_that("sobol_design respects parameter ranges", {
  design <- sobol_design(
    lower = c(x = -10, y = 100),
    upper = c(x = 10, y = 200),
    nseq = 50
  )

  expect_true(all(design$x >= -10 & design$x <= 10))
  expect_true(all(design$y >= 100 & design$y <= 200))
})

test_that("sobol_design handles name ordering", {
  # Different order in lower and upper
  design <- sobol_design(
    lower = c(a = 0, b = 100),
    upper = c(b = 200, a = 1),
    nseq = 10
  )

  expect_equal(names(design), c("a", "b"))
  expect_true(all(design$a >= 0 & design$a <= 1))
  expect_true(all(design$b >= 100 & design$b <= 200))
})

test_that("sobol_design validates inputs", {
  # Length mismatch
  expect_error(
    sobol_design(lower = c(a = 0), upper = c(a = 1, b = 2), nseq = 10),
    "length"
  )

  # Missing names
  expect_error(
    sobol_design(lower = c(0, 1), upper = c(1, 2), nseq = 10),
    "Assertion on 'lower' failed: Must have names."
  )

  # Name mismatch
  expect_error(
    sobol_design(lower = c(a = 0, b = 1), upper = c(a = 1, c = 2), nseq = 10),
    "names"
  )

  # Invalid nseq
  expect_error(
    sobol_design(lower = c(a = 0), upper = c(a = 1), nseq = -1),
    "nseq"
  )

  expect_error(
    sobol_design(lower = c(a = 0), upper = c(a = 1), nseq = 1.5),
    "nseq"
  )

  # Too many points
  expect_error(
    sobol_design(lower = c(a = 0), upper = c(a = 1), nseq = 2^30 + 1),
    "Assertion on 'nseq' failed"
  )

  # Empty vectors
  expect_error(
    sobol_design(lower = numeric(0), upper = numeric(0), nseq = 10),
    "Must have length >= 1, "
  )

  # Lower >= upper
  expect_error(
    sobol_design(lower = c(a = 1), upper = c(a = 0), nseq = 10),
    "lower.*less than"
  )

  # Non-finite values
  expect_error(
    sobol_design(lower = c(a = 0), upper = c(a = Inf), nseq = 10),
    "finite"
  )
})

test_that("sobol_design works with high dimensions", {
  params <- paste0("p", 1:10)
  design <- sobol_design(
    lower = setNames(rep(0, 10), params),
    upper = setNames(rep(1, 10), params),
    nseq = 100
  )

  expect_equal(nrow(design), 100)
  expect_equal(ncol(design), 10)
  expect_equal(names(design), params)
  expect_true(all(design >= 0 & design <= 1))
})

test_that("sobol_design is reproducible", {
  design1 <- sobol_design(
    lower = c(a = 0, b = 0),
    upper = c(a = 1, b = 1),
    nseq = 100
  )

  design2 <- sobol_design(
    lower = c(a = 0, b = 0),
    upper = c(a = 1, b = 1),
    nseq = 100
  )

  expect_equal(design1, design2)
})

test_that("sobol_design produces valid Sobol sequence properties", {
  # Generate design
  design <- sobol_design(
    lower = c(x = 0, y = 0),
    upper = c(x = 1, y = 1),
    nseq = 1000
  )

  # All points should be in [0, 1] range
  expect_true(all(design$x >= 0 & design$x <= 1))
  expect_true(all(design$y >= 0 & design$y <= 1))

  # Coverage test: all quadrants should be represented
  q1 <- sum(design$x < 0.5 & design$y < 0.5)
  q2 <- sum(design$x >= 0.5 & design$y < 0.5)
  q3 <- sum(design$x < 0.5 & design$y >= 0.5)
  q4 <- sum(design$x >= 0.5 & design$y >= 0.5)

  # Each quadrant should have roughly 25% of points
  expect_true(q1 > 200 && q1 < 300)
  expect_true(q2 > 200 && q2 < 300)
  expect_true(q3 > 200 && q3 < 300)
  expect_true(q4 > 200 && q4 < 300)

  # Check low discrepancy: points should be well-distributed
  # Star discrepancy should be low for Sobol sequences
  # For 1000 points in 2D, we expect good coverage
  expect_equal(nrow(design), 1000)
  expect_equal(ncol(design), 2)
})

test_that("sobol_design API matches pomp-explore expectations", {
  # This test validates that the API is compatible with pomp-explore's sobol_design

  # Example from pomp-explore tests
  design <- sobol_design(
    lower = c(a = 0, b = 100),
    upper = c(b = 200, a = 1),
    nseq = 100
  )

  # Should return data frame
  expect_s3_class(design, "data.frame")

  # Should have correct dimensions
  expect_equal(nrow(design), 100)
  expect_equal(ncol(design), 2)

  # Should preserve parameter names from lower
  expect_equal(names(design), c("a", "b"))

  # Should scale correctly
  expect_true(all(design$a >= 0 & design$a <= 1))
  expect_true(all(design$b >= 100 & design$b <= 200))
})

test_that("sobol_design matches pomp-explore reference values", {
  # Test case 1: nseq = 4
  # Expected values from pomp-explore (sorted):
  # [0.250, 0.375, 0.750, 0.875]
  design_4 <- sobol_design(
    lower = c(p = 0),
    upper = c(p = 1),
    nseq = 4
  )

  sorted_4 <- sort(design_4$p)
  expected_4 <- c(0.250, 0.375, 0.750, 0.875)

  # Check values match (with tolerance for floating point)
  expect_equal(sorted_4, expected_4, tolerance = 1e-10)

  # Test case 2: nseq = 10
  # Expected values from pomp-explore (sorted):
  # [0.06250, 0.09375, 0.18750, 0.31250, 0.43750, 0.56250,
  #  0.59375, 0.68750, 0.81250, 0.93750]
  design_10 <- sobol_design(
    lower = c(p = 0),
    upper = c(p = 1),
    nseq = 10
  )

  sorted_10 <- sort(design_10$p)
  expected_10 <- c(
    0.06250, 0.09375, 0.18750, 0.31250, 0.43750,
    0.56250, 0.59375, 0.68750, 0.81250, 0.93750
  )

  # Check values match (with tolerance for floating point)
  expect_equal(sorted_10, expected_10, tolerance = 1e-10)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_index}
\alias{sobol_index}
\title{Get Current Index of a Sobol Generator}
\usage{
sobol_index(x, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{...}{Additional arguments (currently unused)}
}
\value{
A numeric value representing the current index (0-based).
}
\description{
Returns the current index of the generator, which is the index of the
next point that will be generated.
}
\examples{
gen <- sobol_generator(dimensions = 2)
sobol_next(gen)
sobol_next(gen)
idx <- sobol_index(gen)
print(idx) # 2

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_dimensions}
\alias{sobol_dimensions}
\title{Get Number of Dimensions of a Sobol Generator}
\usage{
sobol_dimensions(x, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{...}{Additional arguments (currently unused)}
}
\value{
An integer representing the number of dimensions.
}
\description{
Returns the number of dimensions configured for the generator.
}
\examples{
gen <- sobol_generator(dimensions = 5)
dims <- sobol_dimensions(gen)
print(dims) # 5

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_points.R
\name{sobol_points}
\alias{sobol_points}
\title{Generate a Batch of Sobol Points}
\arguments{
\item{n}{Integer, the number of points to generate. Must be non-negative.}

\item{dim}{Integer, the number of dimensions for each point.
Must be a positive integer.}

\item{skip}{Numeric, the number of initial points to skip (default: 0).
This allows generating subsequences of the Sobol sequence.}
}
\value{
A numeric matrix with n rows and dim columns, where each
  row represents a point in the Sobol sequence. Values are in [0, 1).
  If n = 0, returns a 0 x dim matrix.
}
\description{
Efficiently generates a matrix of Sobol sequence points. This is a
convenience function that does not maintain state between calls.
For incremental generation, use \code{\link{sobol_generator}} instead.
}
\details{
This function is implemented directly in C++ via Rcpp and is more
efficient than creating a generator and calling \code{sobol_next_n}
when you know in advance how many points you need.

The skip parameter allows you to start from any point in the sequence,
which is useful for:
\itemize{
  \item Reproducibility: generating the same subsequence across runs
  \item Parallelization: different workers can generate
    non-overlapping segments
  \item Continuation: extending a previous sequence without
    regenerating early points
}
}
\examples{
# Generate 1000 points in 5 dimensions
points <- sobol_points(n = 1000, dim = 5)
dim(points) # [1] 1000    5

# Skip the first 100 points
points_skipped <- sobol_points(n = 100, dim = 2, skip = 100)

# Empty result
empty <- sobol_points(n = 0, dim = 3)
dim(empty) # [1] 0 3

}
\seealso{
\code{\link{sobol_generator}} for incremental generation with state
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{summary.sobol_generator}
\alias{summary.sobol_generator}
\title{Summary Method for Sobol Generator}
\usage{
\method{summary}{sobol_generator}(object, ...)
}
\arguments{
\item{object}{A sobol_generator object}

\item{...}{Additional arguments passed to summary}
}
\value{
A list with class "summary.sobol_generator" containing summary
information
}
\description{
Summary Method for Sobol Generator
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol-package.R
\docType{package}
\name{sobol-package}
\alias{sobol}
\alias{sobol-package}
\title{sobol: Quasi-Monte Carlo Sobol Sequence Generator}
\description{
Provides a fast and efficient implementation of Sobol sequences for quasi-Monte Carlo methods. The Sobol sequence is a low-discrepancy sequence with the property that for all values of N, its subsequence x1, ..., xN has a low discrepancy. It can be used to generate quasi-random numbers for use in Monte Carlo integration and other simulation methods. This implementation is based on the algorithms described by Bratley and Fox (1988) \doi{10.1145/42288.214372} and uses direction numbers from Joe and Kuo (2008) \doi{10.1145/1358628.1358630}. The package includes both batch and incremental interfaces with support for arbitrary starting indices and reproducible sequences. It uses 'Rcpp' for efficient 'C++' integration.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://alrobles.github.io/sobol/}
  \item Report bugs at \url{https://github.com/alrobles/sobol/issues}
}

}
\author{
\strong{Maintainer}: Angel Robles \email{a.l.robles.fernandez@gmail.com}

Other contributors:
\itemize{
  \item Ilya M. Sobol (Original Sobol sequence algorithm) [contributor]
  \item Paul Bratley (Algorithm implementation reference) [contributor]
  \item Bennett L. Fox (Algorithm implementation reference) [contributor]
  \item Stephen Joe (Direction numbers and primitive polynomials) [contributor]
  \item Frances Y. Kuo (Direction numbers and primitive polynomials) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_generator}
\alias{sobol_generator}
\title{Create a Sobol Sequence Generator}
\usage{
sobol_generator(dimensions, skip = 0)
}
\arguments{
\item{dimensions}{Integer, the number of dimensions for the Sobol sequence.
Must be a positive integer.}

\item{skip}{Numeric, the number of initial points to skip (default: 0).
This allows starting the sequence from any index for reproducibility.}
}
\value{
An S3 object of class "sobol_generator" containing:
  \item{generator}{The underlying Rcpp reference class object}
  \item{dimensions}{Number of dimensions}
  \item{initial_skip}{Initial skip value used at construction}
}
\description{
Creates an S3 object that wraps an Rcpp Sobol sequence generator for
incremental point generation. The generator maintains internal state
and allows for efficient generation of quasi-random sequences.
}
\examples{
# Create a 3-dimensional Sobol generator
gen <- sobol_generator(dimensions = 3)

# Generate a single point
point <- sobol_next(gen)

# Generate multiple points
points <- sobol_next_n(gen, n = 100)

# Skip to a specific index
sobol_skip_to(gen, 1000)

# Create a generator starting from index 50
gen2 <- sobol_generator(dimensions = 2, skip = 50)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_design.R
\name{sobol_design}
\alias{sobol_design}
\title{Generate a Sobol Design for Parameter Exploration}
\usage{
sobol_design(lower = numeric(0), upper = numeric(0), nseq)
}
\arguments{
\item{lower}{Named numeric vector giving the lower bounds of the parameter
ranges. Must have the same names as \code{upper}.}

\item{upper}{Named numeric vector giving the upper bounds of the parameter
ranges. Must have the same names as \code{lower}.}

\item{nseq}{Integer, the total number of parameter sets (points) to generate.}
}
\value{
A data frame with \code{nseq} rows and one column for each parameter
  named in \code{lower} and \code{upper}. Each column contains values scaled
  to the specified range [lower, upper] for that parameter.
}
\description{
Creates a Latin hypercube design based on the Sobol low-discrepancy sequence.
This function provides an API-compatible alternative to the sobol_design
function in the pomp-explore package for generating parameter designs.
}
\details{
This function generates a Sobol sequence in the unit hypercube [0,1]^d and
then scales each dimension to the specified parameter ranges. The Sobol
sequence is generated using the Joe-Kuo direction numbers with Property A
enforcement, providing excellent low-discrepancy properties.

Following the recommendation of Joe & Kuo (2003) and the implementation
in pomp-explore, this function skips the first k points of the Sobol
sequence, where k is the largest power of 2 smaller than nseq. This
improves the uniformity properties of the generated design.

The function is designed to be API-compatible with the \code{sobol_design}
function from the pomp-explore package, allowing for easy comparison and
drop-in replacement.
}
\examples{
# Generate 100 parameter sets for two parameters
design <- sobol_design(
  lower = c(a = 0, b = 100),
  upper = c(a = 1, b = 200),
  nseq = 100
)
head(design)

# Plot the design
plot(design$a, design$b, main = "Sobol Design")

# High-dimensional example
params <- paste0("param", 1:10)
design_10d <- sobol_design(
  lower = setNames(rep(0, 10), params),
  upper = setNames(rep(1, 10), params),
  nseq = 1000
)

}
\references{
Bratley, P., & Fox, B. L. (1988). Algorithm 659: Implementing Sobol's
quasirandom sequence generator. ACM Transactions on Mathematical Software,
14(1), 88-100.

Joe, S., & Kuo, F. Y. (2008). Constructing Sobol sequences with better
two-dimensional projections. SIAM Journal on Scientific Computing, 30(5),
2635-2654.
}
\seealso{
\code{\link{sobol_points}} for batch generation without scaling,
  \code{\link{sobol_generator}} for incremental generation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{print.summary.sobol_generator}
\alias{print.summary.sobol_generator}
\title{Print Summary of Sobol Generator}
\usage{
\method{print}{summary.sobol_generator}(x, ...)
}
\arguments{
\item{x}{A summary.sobol_generator object}

\item{...}{Additional arguments passed to print}
}
\value{
Invisibly returns the input object
}
\description{
Print Summary of Sobol Generator
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_next_n}
\alias{sobol_next_n}
\title{Generate Multiple Points from a Sobol Generator}
\usage{
sobol_next_n(x, n, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{n}{Integer, the number of points to generate. Must be non-negative.}

\item{...}{Additional arguments (currently unused)}
}
\value{
A numeric matrix with n rows and dimensions columns, where each
  row represents a point in the Sobol sequence. Values are in [0, 1).
  If n = 0, returns a 0 x dimensions matrix.
}
\description{
Generates n consecutive points from the Sobol sequence and advances the
internal state of the generator.
}
\examples{
gen <- sobol_generator(dimensions = 2)
points <- sobol_next_n(gen, n = 10)
print(dim(points)) # [1] 10  2

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_next}
\alias{sobol_next}
\title{Generate the Next Point from a Sobol Generator}
\usage{
sobol_next(x, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{...}{Additional arguments (currently unused)}
}
\value{
A numeric vector of length equal to the number of dimensions,
  containing the next point in the Sobol sequence. Values are in [0, 1).
}
\description{
Generates a single point from the Sobol sequence and advances the
internal state of the generator.
}
\examples{
gen <- sobol_generator(dimensions = 3)
point <- sobol_next(gen)
print(point) # e.g., [0.5, 0.5, 0.5]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{print.sobol_generator}
\alias{print.sobol_generator}
\title{Print Method for Sobol Generator}
\usage{
\method{print}{sobol_generator}(x, ...)
}
\arguments{
\item{x}{A sobol_generator object}

\item{...}{Additional arguments passed to print}
}
\value{
Invisibly returns the input object (for chaining)
}
\description{
Print Method for Sobol Generator
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_skip_to}
\alias{sobol_skip_to}
\title{Skip to a Specific Index in a Sobol Generator}
\usage{
sobol_skip_to(x, index, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{index}{Numeric, the index to skip to. Must be a non-negative integer.
The next call to \code{sobol_next} will return the point at this index.}

\item{...}{Additional arguments (currently unused)}
}
\value{
Invisibly returns the sobol_generator object (for chaining).
  The primary purpose is the side effect of updating the internal state.
}
\description{
Jumps the internal state of the generator to a specific index in the
Sobol sequence. This allows for reproducible subsequences and parallel
generation strategies.
}
\examples{
gen <- sobol_generator(dimensions = 2)
sobol_skip_to(gen, 100)
point <- sobol_next(gen) # This is the 100th point (0-indexed)

}
